#include "xbasic_types.h"
#include "xio.h"

#include "xintc_l.h"
#include "xuartlite_l.h"

#define W 640
#define H 480
#define VGA_START 0x00800000
#define RED 0xE0
#define GREEN 0x1C
#define BLUE 0x03

// defined in isr.c
extern void uart_handler(void *callback);
extern int uart_interrupt_count;
extern char uart_character;

/*
 * setup_interrupts: Initialize the interrupt sources and handlers
 *
 * Should be called once when the system starts
 *
 * The main _interrupt_handler() function from Xilinx
 *
 * Saves and restores CPU context, etc.
 *
 * Sees which interrupts are pending, and for each it
 *     acknowledges the interrupt and
 *     calls a user-defined interrupt handler in Xintc_InterruptVectorTable
 *
 * Place interrupt service routines in isr.c to ensure they are placed in
 * the proper memory segment.
 */
void setup_interrupts()
{
  /*
   * Reset the interrupt controller peripheral
   */

  /* Disable the interrupt signal */
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  /* Disable all interrupt sources */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0);

  /* Acknowledge all possible interrupt sources
     to make sure none are pending */
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff);

  /*
   * Install the UART interrupt handler
   */

  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler =
    uart_handler;

  /*
   * Enable interrupt sources
   */

  /* Enable CPU interrupts */
  microblaze_enable_interrupts();

  /* Enable interrupts from the interrupt controller */
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);

  /* Tell the interrupt controller to accept interrupts from the UART */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_MYUART_INTERRUPT_MASK);
  
  /* Enable UART interrupt generation */
  XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
}

void clear_screen()
{
  int x, y;

  // erasing screen
  for(x=0 ; x<H*W ; x++)
    XIo_Out8(VGA_START + x, 0);

  // red border
  for(x=0 ; x<W ; x++) {
    XIo_Out8(VGA_START + x          , RED);
    XIo_Out8(VGA_START + (H-1)*W + x, RED);
  }
  for(y=0 ; y<H ; y++) {
    XIo_Out8(VGA_START + y*W        , RED);
    XIo_Out8(VGA_START + y*W + W - 1, RED);
  }

  // white lines
  for(x=400 ; x<600 ; x+=20)
    for(y=20; y<100; y++) {
      XIo_Out8(VGA_START + y*W +x  , RED|GREEN|BLUE);
      XIo_Out8(VGA_START + y*W +x+1, RED|GREEN|BLUE);
    }

  // green square
  for(y=100 ; y<150 ; y++)
    for(x=100 ; x<150 ; x++)
      XIo_Out8(VGA_START + y*W + x, GREEN);

  // blue circle
  for(y=-100 ; y<100 ; y++)
    for(x=0 ; x*x+y*y<10000 ; x++) {
      XIo_Out8(VGA_START + (y+280)*W + x + 400, BLUE);
      XIo_Out8(VGA_START + (y+280)*W - x + 400, BLUE);
    }

  // YELLOW TRIANGLE
  for(y=0 ; y<100 ; y++)
    for(x=-y ; x<y ; x++)
      XIo_Out8(VGA_START + (y+310)*W + x + 150, GREEN | RED);
}

int main()
{
  char c;
  int i;
  char buf[2];
  buf[1] = '\0';

  // Enable the instruction cache: makes the code run 6 times faster
  microblaze_enable_icache();

  print("Hello Lab3!\r\n");

  setup_interrupts();

  clear_screen();
  
  for (;;) {

    print("Character number ");
    putnum(uart_interrupt_count);
    print(" is ");
    buf[0] = uart_character;
    print(buf);
    print("\r\n");
    
    for (i = 0 ; i < 10000000 ; i++ )  ; /* delay */
  }

  return 0;
}
